import myparser
import requests
import sys

class search_hunter:

    def __init__(self, word, limit, start):
        self.word = word
        self.limit = 100
        self.start = start
        self.key = ""
        if self.key == "":
            print("You need an API key in order to use the Hunter search engine. You can get one here: https://hunter.io")
            sys.exit()
        self.results = ""
        self.totalresults = ""
        self.counter = start
        self.database = "https://api.hunter.io/v2/domain-search?domain=" + word + "&api_key=" + self.key +"&limit=" + str(self.limit)

    def do_search(self):
        try:
            r = requests.get(self.database)
        except Exception as e:
            print(e)
        self.results = r.text
        self.totalresults += self.results

    def process(self):
            self.do_search() #only need to do it once
            print('\tDone Searching Results')

    def get_emails(self):
        rawres = myparser.parser(self.totalresults, self.word)
        return rawres.emails()

    def get_hostnames(self):
        rawres = myparser.parser(self.totalresults, self.word)
        return rawres.hostnames()

    def get_profiles(self):
        rawres = myparser.parser(self.totalresults, self.word)
        return rawres.profiles()



#!/usr/bin/env python
'''
F-Secure Virus World Map console edition

See README.md for more details

Copyright 2012-2013 Jyrki Muukkonen

Released under the MIT license.
See LICENSE.txt or http://www.opensource.org/licenses/mit-license.php

ASCII map in map-world-01.txt is copyright:
 "Map 1998 Matthew Thomas. Freely usable as long as this line is included"

'''
import curses
import locale
import random
import time

from discovery.shodan.helpers import get_ip


MAPS = {
    'world': {
        # offset (as (y, x) for curses...)
        'corners': (1, 4, 23, 73),
        # lat top, lon left, lat bottom, lon right
        'coords': [90.0, -180.0, -90.0, 180.0],
        'data': '''
               . _..::__:  ,-"-"._       |7       ,     _,.__             
       _.___ _ _<_>`!(._`.`-.    /        _._     `_ ,_/  '  '-._.---.-.__
     .{     " " `-==,',._\{  \  / {)     / _ ">_,-' `                mt-2_
      \_.:--.       `._ )`^-. "'      , [_/(                       __,/-' 
     '"'     \         "    _L       oD_,--'                )     /. (|   
              |           ,'         _)_.\\._<> 6              _,' /  '   
              `.         /          [_/_'` `"(                <'}  )      
               \\    .-. )          /   `-'"..' `:._          _)  '       
        `        \  (  `(          /         `:\  > \  ,-^.  /' '         
                  `._,   ""        |           \`'   \|   ?_)  {\         
                     `=.---.       `._._       ,'     "`  |' ,- '.        
                       |    `-._        |     /          `:`<_|h--._      
                       (        >       .     | ,          `=.__.`-'\     
                        `.     /        |     |{|              ,-.,\     .
                         |   ,'          \   / `'            ,"     \     
                         |  /             |_'                |  __  /     
                         | |                                 '-'  `-'   \.
                         |/                                        "    / 
                         \.                                            '  
                                                                          
                          ,/           ______._.--._ _..---.---------._   
         ,-----"-..?----_/ )      _,-'"             "                  (  
    Map 1998 Matthew Thomas. Freely usable as long as this line is included
'''
    }
}


class AsciiMap(object):
    """
    Helper class for handling map drawing and coordinate calculations
    """
    def __init__(self, map_name='world', map_conf=None, window=None, encoding=None):
        if map_conf is None:
            map_conf = MAPS[map_name]
        self.map = map_conf['data']
        self.coords = map_conf['coords']
        self.corners = map_conf['corners']
        if window is None:
            window = curses.newwin(0, 0)
        self.window = window

        self.data = []
        self.data_timestamp = None

        # JSON contents _should_ be UTF8 (so, python internal unicode here...)
        if encoding is None:
            encoding = locale.getpreferredencoding()
        self.encoding = encoding

        # check if we can use transparent background or not
        if curses.can_change_color():
            curses.use_default_colors()
            background = -1
        else:
            background = curses.COLOR_BLACK

        tmp_colors = [
            ('red', curses.COLOR_RED, background),
            ('blue', curses.COLOR_BLUE, background),
            ('pink', curses.COLOR_MAGENTA, background)
        ]

        self.colors = {}
        if curses.has_colors():
            for i, (name, fgcolor, bgcolor) in enumerate(tmp_colors, 1):
                curses.init_pair(i, fgcolor, bgcolor)
                self.colors[name] = i

    def latlon_to_coords(self, lat, lon):
        """
        Convert lat/lon coordinates to character positions.
        Very naive version, assumes that we are drawing the whole world
        TODO: filter out stuff that doesn't fit
        TODO: make it possible to use "zoomed" maps
        """
        width = (self.corners[3]-self.corners[1])
        height = (self.corners[2]-self.corners[0])

        # change to 0-180, 0-360
        abs_lat = -lat+90
        abs_lon = lon+180
        x = (abs_lon/360.0)*width + self.corners[1]
        y = (abs_lat/180.0)*height + self.corners[0]
        return int(x), int(y)

    def set_data(self, data):
        """
        Set / convert internal data.
        For now it just selects a random set to show.
        """
        entries = []

        # Grab 5 random banners to display
        for banner in random.sample(data, min(len(data), 5)):
            desc = '{} -> {} / {}'.format(get_ip(banner), banner['port'], banner['location']['country_code'])
            if banner['location']['city']:
                desc += ' {}'.format(banner['location']['city'])
            
            if 'tags' in banner and banner['tags']:
                desc += ' / {}'.format(','.join(banner['tags']))
            
            entry = (
                float(banner['location']['latitude']),
                float(banner['location']['longitude']),
                '*',
                desc,
                curses.A_BOLD,
                'red',
            )
            entries.append(entry)
        self.data = entries

    def draw(self, target):
        """ Draw internal data to curses window """
        self.window.clear()
        self.window.addstr(0, 0, self.map)

        # FIXME: position to be defined in map config?
        row = self.corners[2]-6
        items_to_show = 5
        for lat, lon, char, desc, attrs, color in self.data:
            # to make this work almost everywhere. see http://docs.python.org/2/library/curses.html
            if desc:
                desc = desc.encode(self.encoding, 'ignore')
            if items_to_show <= 0:
                break
            char_x, char_y = self.latlon_to_coords(lat, lon)
            if self.colors and color:
                attrs |= curses.color_pair(self.colors[color])
            self.window.addstr(char_y, char_x, char, attrs)
            if desc:
                det_show = "%s %s" % (char, desc)
            else:
                det_show = None

            if det_show is not None:
                try:
                    self.window.addstr(row, 1, det_show, attrs)
                    row += 1
                    items_to_show -= 1
                except Exception:
                    # FIXME: check window size before addstr()
                    break
        self.window.overwrite(target)
        self.window.leaveok(1)


class MapApp(object):
    """ Virus World Map ncurses application """
    def __init__(self, api):
        self.api = api
        self.data = None
        self.last_fetch = 0
        self.sleep = 10  # tenths of seconds, for curses.halfdelay()
        self.polling_interval = 60

    def fetch_data(self, epoch_now, force_refresh=False):
        """ (Re)fetch data from JSON stream """
        refresh = False
        if force_refresh or self.data is None:
            refresh = True
        else:
            if self.last_fetch + self.polling_interval <= epoch_now:
                refresh = True
        
        if refresh:
            try:
                # Grab 20 banners from the main stream
                banners = []
                for banner in self.api.stream.banners():
                    if 'location' in banner and banner['location']['latitude']:
                        banners.append(banner)
                    if len(banners) >= 20:
                        break
                self.data = banners
                self.last_fetch = epoch_now
            except Exception:
                raise
        return refresh

    def run(self, scr):
        """ Initialize and run the application """
        m = AsciiMap()
        curses.halfdelay(self.sleep)
        while True:
            now = int(time.time())
            refresh = self.fetch_data(now)
            m.set_data(self.data)
            m.draw(scr)
            scr.addstr(0, 1, 'Shodan Radar', curses.A_BOLD)
            scr.addstr(0, 40, time.strftime("%c UTC", time.gmtime(now)).rjust(37), curses.A_BOLD)

            # Key Input
            # q     - Quit
            event = scr.getch()
            if event == ord('q'):
                break

            # redraw window (to fix encoding/rendering bugs and to hide other messages to same tty)
            # user pressed 'r' or new data was fetched
            if refresh:
                m.window.redrawwin()


def launch_map(api):
    app = MapApp(api)
    return curses.wrapper(app.run)


def main(argv=None):
    """ Main function / entry point """
    from discovery.shodan import Shodan
    from discovery.shodan.cli.helpers import get_api_key

    api = Shodan(get_api_key())
    return launch_map(api)

if __name__ == '__main__':
    import sys
    sys.exit(main())



"""
 $Id: win32dns.py,v 1.3.2.1 2007/05/22 20:26:49 customdesigned Exp $

 Extract a list of TCP/IP name servers from the registry 0.1
    0.1 Strobl 2001-07-19
 Usage:
    RegistryResolve() returns a list of ip numbers (dotted quads), by
    scouring the registry for addresses of name servers

 Tested on Windows NT4 Server SP6a, Windows 2000 Pro SP2 and
 Whistler Pro (XP) Build 2462 and Windows ME
 ... all having a different registry layout wrt name servers :-/

 Todo:

   Program doesn't check whether an interface is up or down

 (c) 2001 Copyright by Wolfgang Strobl ws@mystrobl.de,
          License analog to the current Python license
"""

import winreg

def binipdisplay(s):
    "convert a binary array of ip adresses to a python list"
    if len(s) % 4 != 0:
        raise EnvironmentError  # well ...
    ol = []
    s = '.'
    for i in range(int(len(s) / 4)):
        s1 = s[:4]
        s = s[4:]
        ip = []
        for j in s1:
            ip.append(str(ord(j)))
        ol.append(s.join(ip))
    return ol


def stringdisplay(s):
    '''convert "d.d.d.d,d.d.d.d" to ["d.d.d.d","d.d.d.d"].
       also handle u'd.d.d.d d.d.d.d', as reporting on SF
    '''
    import re
    return map(str, re.split("[ ,]", s))


def RegistryResolve():
    nameservers = []
    x = winreg.ConnectRegistry(None, winreg.HKEY_LOCAL_MACHINE)
    try:
        y = winreg.OpenKey(x,
                            r"SYSTEM\CurrentControlSet\Services\Tcpip\Parameters")
    except EnvironmentError:  # so it isn't NT/2000/XP
        # windows ME, perhaps?
        try:  # for Windows ME
            y = winreg.OpenKey(x,
                                r"SYSTEM\CurrentControlSet\Services\VxD\MSTCP")
            nameserver, dummytype = winreg.QueryValueEx(y, 'NameServer')
            if nameserver and not (nameserver in nameservers):
                nameservers.extend(stringdisplay(nameserver))
        except EnvironmentError:
            pass
        return nameservers  # no idea
    try:
        nameserver = winreg.QueryValueEx(y, "DhcpNameServer")[0].split()
    except:
        nameserver = winreg.QueryValueEx(y, "NameServer")[0].split()
    if nameserver:
        nameservers = nameserver
    nameserver = winreg.QueryValueEx(y, "NameServer")[0]
    winreg.CloseKey(y)
    try:  # for win2000
        y = winreg.OpenKey(x,
                            r"SYSTEM\CurrentControlSet\Services\Tcpip\Parameters\DNSRegisteredAdapters")
        for i in range(1000):
            try:
                n = winreg.EnumKey(y, i)
                z = winreg.OpenKey(y, n)
                dnscount, dnscounttype = winreg.QueryValueEx(z,
                                                              'DNSServerAddressCount')
                dnsvalues, dnsvaluestype = winreg.QueryValueEx(z,
                                                                'DNSServerAddresses')
                nameservers.extend(binipdisplay(dnsvalues))
                winreg.CloseKey(z)
            except EnvironmentError:
                break
        winreg.CloseKey(y)
    except EnvironmentError:
        pass
#
    try:  # for whistler
        y = winreg.OpenKey(x,
                            r"SYSTEM\CurrentControlSet\Services\Tcpip\Parameters\Interfaces")
        for i in range(1000):
            try:
                n = winreg.EnumKey(y, i)
                z = winreg.OpenKey(y, n)
                try:
                    nameserver, dummytype = winreg.QueryValueEx(
                        z, 'NameServer')
                    if nameserver and not (nameserver in nameservers):
                        nameservers.extend(stringdisplay(nameserver))
                except EnvironmentError:
                    pass
                winreg.CloseKey(z)
            except EnvironmentError:
                break
        winreg.CloseKey(y)
    except EnvironmentError:
        # print "Key Interfaces not found, just do nothing"
        pass
#
    winreg.CloseKey(x)
    return nameservers

if __name__ == "__main__":
    print("Name servers:", RegistryResolve())

#
# $Log: win32dns.py,v $
# Revision 1.3.2.1  2007/05/22 20:26:49  customdesigned
# Fix win32 nameserver discovery.
#
# Revision 1.3  2002/05/06 06:15:31  anthonybaxter
# apparently some versions of windows return servers as unicode
# string with space sep, rather than strings with comma sep.
# *sigh*
#
# Revision 1.2  2002/03/19 12:41:33  anthonybaxter
# tabnannied and reindented everything. 4 space indent, no tabs.
# yay.
#
# Revision 1.1  2001/08/09 09:22:28  anthonybaxter
# added what I hope is win32 resolver lookup support. I'll need to try
# and figure out how to get the CVS checkout onto my windows machine to
# make sure it works (wow, doing something other than games on the
# windows machine :)
#
# Code from Wolfgang.Strobl@gmd.de
# win32dns.py from
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66260
#
# Really, ParseResolvConf() should be renamed "FindNameServers" or
# some such.
#
#




from .base import Converter
from ...helpers import iterate_files

from collections import MutableMapping
from csv import writer as csv_writer, excel


class CsvConverter(Converter):

    fields = [
        'data',
        'hostnames',
        'ip',
        'ip_str',
        'ipv6',
        'org',
        'isp',
        'location.country_code',
        'location.city',
        'location.country_name',
        'location.latitude',
        'location.longitude',
        'os',
        'asn',
        'port',
        'transport',
        'product',
        'version',
        
        'ssl.cipher.version',
        'ssl.cipher.bits',
        'ssl.cipher.name',
        'ssl.alpn',
        'ssl.versions',
        'ssl.cert.serial',
        'ssl.cert.fingerprint.sha1',
        'ssl.cert.fingerprint.sha256',
        
        'html',
        'title',
    ]
    
    def process(self, files):
        writer = csv_writer(self.fout, dialect=excel)
        
        # Write the header
        writer.writerow(self.fields)
        
        for banner in iterate_files(files):
            try:
                row = []
                for field in self.fields:
                    value = self.banner_field(banner, field)
                    row.append(value)
                writer.writerow(row)
            except:
                pass
    
    def banner_field(self, banner, flat_field):
        # The provided field is a collapsed form of the actual field
        fields = flat_field.split('.')
    
        try:
            current_obj = banner
            for field in fields:
                current_obj = current_obj[field]
            
            # Convert a list into a concatenated string
            if isinstance(current_obj, list):
                current_obj = ','.join([str(i) for i in current_obj])
            
            return current_obj
        except:
            pass
    
        return ''
    
    def flatten(self, d, parent_key='', sep='.'):
        items = []
        for k, v in d.items():
            new_key = parent_key + sep + k if parent_key else k
            if isinstance(v, MutableMapping):
                items.extend(flatten(v, new_key, sep=sep).items())
            else:
                items.append((new_key, v))
        return dict(items)


from lib import markup
from lib import graphs
import re


class htmlExport():

    def __init__(self, users, hosts, vhosts, dnsres,
                 dnsrev, file, domain, shodan, tldres):
        self.users = users
        self.hosts = hosts
        self.vhost = vhosts
        self.fname = file
        self.dnsres = dnsres
        self.dnsrev = dnsrev
        self.domain = domain
        self.shodan = shodan
        self.tldres = tldres
        self.style = ""

    def styler(self):
        a = """<style type='text/css'>body {
			 background: #FFFFFF  top no-repeat;
		 }

		h1 { font-family: arial, Times New Roman, times-roman, georgia, serif;
			color: #680000;
			margin: 0;
			padding: 0px 0px 6px 0px;
			font-size: 51px;
			line-height: 44px;
			letter-spacing: -2px;
			font-weight: bold;
		}

		h3 { font-family: arial, Times New Roman, times-roman, georgia, serif;
			color: #444;
			margin: 0;
			padding: 0px 0px 6px 0px;
			font-size: 30px;
			line-height: 44px;
			letter-spacing: -2px;
			font-weight: bold;
		}

		li { font-family: arial, Times New Roman, times-roman, georgia, serif;
			color: #444;
			margin: 0;
			padding: 0px 0px 6px 0px;
			font-size: 15px;
			line-height: 15px;
			letter-spacing: 0.4px;

		}

		h2{
		font-family: arial, Times New Roman, times-roman, georgia, serif;
				font-size: 48px;
				line-height: 40px;
				letter-spacing: -1px;
				color: #680000 ;
				margin: 0 0 0 0;
				padding: 0 0 0 0;
				font-weight: 100;

		}

		pre {
		overflow: auto;
		padding-left: 15px;
		padding-right: 15px;
		font-size: 11px;
		line-height: 15px;
		margin-top: 10px;
		width: 93%;
		display: block;
		background-color: #eeeeee;
		color: #000000;
		max-height: 300px;
		}
		</style>
		"""
        self.style = a

    def writehtml(self):
        page = markup.page()
        # page.init (title="theHarvester
        # Results",css=('edge.css'),footer="Edge-security 2011")A
        page.html()
        self.styler()
        page.head(self.style)
        page.body()
        page.h1("theHarvester results")
        page.h2("for :" + self.domain)
        page.h3("Dashboard:")
        graph = graphs.BarGraph('vBar')
        graph.values = [len(
            self.users),
            len(self.hosts),
            len(self.vhost),
            len(self.tldres),
            len(self.shodan)]
        graph.labels = ['Emails', 'hosts', 'Vhost', 'TLD', 'Shodan']
        graph.showValues = 1
        page.body(graph.create())
        page.h3("E-mails names found:")
        if self.users != []:
            page.ul(class_="userslist")
            page.li(self.users, class_="useritem")
            page.ul.close()
        else:
            page.h2("No emails found")
        page.h3("Hosts found:")
        if self.hosts != []:
            page.ul(class_="softlist")
            page.li(self.hosts, class_="softitem")
            page.ul.close()
        else:
            page.h2("No hosts found")
        if self.tldres != []:
            page.h3("TLD domains found in TLD expansion:")
            page.ul(class_="tldlist")
            page.li(self.tldres, class_="tlditem")
            page.ul.close()
        if self.dnsres != []:
            page.h3("Hosts found in DNS brute force:")
            page.ul(class_="dnslist")
            page.li(self.dnsres, class_="dnsitem")
            page.ul.close()
        if self.dnsrev != []:
            page.h3("Hosts found with reverse lookup :")
            page.ul(class_="dnsrevlist")
            page.li(self.dnsrev, class_="dnsrevitem")
            page.ul.close()
        if self.vhost != []:
            page.h3("Virtual hosts found:")
            page.ul(class_="pathslist")
            page.li(self.vhost, class_="pathitem")
            page.ul.close()
        if self.shodan != []:
            shodanalysis = []
            page.h3("Shodan results:")
            for x in self.shodan:
                res = x.split("SAPO")
                page.h3(res[0])
                page.a("Port :" + res[2])
                page.pre(res[1])
                page.pre.close()
                ban = res[1]
                reg_server = re.compile('Server:.*')
                temp = reg_server.findall(res[1])
                if temp != []:
                    shodanalysis.append(res[0] + ":" + temp[0])
            if shodanalysis != []:
                page.h3("Server technologies:")
                repeated = []
                for x in shodanalysis:
                    if x not in repeated:
                        page.pre(x)
                        page.pre.close()
                        repeated.append(x)
        page.body.close()
        page.html.close()
        file = open(self.fname, 'w')
        for x in page.content:
            try:
                file.write(x)
            except:
                print("Exception" + x) # send to logs
                pass
        file.close()
        return "ok"



import requests
import myparser
from discovery.constants import *

class search_netcraft:

    def __init__(self, word):
        self.word = word.replace(' ', '%20')
        self.results = ""
        self.totalresults = ""
        self.server = "www.google.com"
        self.hostname = "www.google.com"
        self.quantity = "100"
        self.counter = 0
        

    def do_search(self):
        try:
            urly="https://searchdns.netcraft.com/?restriction=site+ends+with&host=" + self.word
        except Exception as e:
            print(e)
        headers = {'User-Agent':getUserAgent()}
        try:
            r=requests.get(urly,headers=headers)
        except Exception as e:
            print(e)
        self.results = r.text
        self.totalresults += self.results

    def get_hostnames(self):
        rawres = myparser.parser(self.results, self.word)
        return rawres.hostnames()

    def process(self):
        self.do_search()
        print("\tSearching Netcraft results..")



# -*- encoding: utf-8 -*-
"""
 $Id: Lib.py,v 1.11.2.3 2007/05/22 20:27:40 customdesigned Exp $

 This file is part of the pydns project.
 Homepage: http://pydns.sourceforge.net

 This code is covered by the standard Python License.

 Library code. Largely this is packers and unpackers for various types.
"""

#
#
# See RFC 1035:
# ------------------------------------------------------------------------
# Network Working Group                                     P. Mockapetris
# Request for Comments: 1035                                           ISI
#                                                            November 1987
# Obsoletes: RFCs 882, 883, 973
#
#             DOMAIN NAMES - IMPLEMENTATION AND SPECIFICATION
# ------------------------------------------------------------------------

from discovery.DNS import Type, Class, Opcode, Status
from discovery.DNS.Base import DNSError

class UnpackError(DNSError):
    pass


class PackError(DNSError):
    pass

# Low-level 16 and 32 bit integer packing and unpacking

from struct import pack as struct_pack
from struct import unpack as struct_unpack
from socket import inet_ntoa, inet_aton


def pack16bit(n):
    return struct_pack('!H', n)


def pack32bit(n):
    return struct_pack('!L', n)


def unpack16bit(s):
    return struct_unpack('!H', s)[0]


def unpack32bit(s):
    return struct_unpack('!L', s)[0]


def addr2bin(addr):
    return struct_unpack('!l', inet_aton(addr))[0]


def bin2addr(n):
    return inet_ntoa(struct_pack('!L', n))

# Packing class


class Packer:

    " packer base class. supports basic byte/16bit/32bit/addr/string/name "

    def __init__(self):
        self.buf = b''
        self.index = {}

    def getbuf(self):
        return self.buf

    def addbyte(self, c):
        if len(c) != 1:
            raise TypeError('one character expected')
        self.buf = self.buf + c

    def addbytes(self, bytes):
        self.buf = self.buf + bytes

    def add16bit(self, n):
        self.buf = self.buf + pack16bit(n)

    def add32bit(self, n):
        self.buf = self.buf + pack32bit(n)

    def addaddr(self, addr):
        n = addr2bin(addr)
        self.buf = self.buf + pack32bit(n)

    def addstring(self, s):
        if len(s) > 255:
            raise ValueError("Can't encode string of length " +
                             "%s (> 255)" % (len(s)))
        self.addbyte(chr(len(s)))
        self.addbytes(s)

    def addname(self, name):
        # Domain name packing (section 4.1.4)
        # Add a domain name to the buffer, possibly using pointers.
        # The case of the first occurrence of a name is preserved.
        # Redundant dots are ignored.
        list = []
        for label in name.split('.'):
            if label:
                if len(label) > 63:
                    raise PackError('label too long')
                list.append(label)
        keys = []
        s = ''
        for i in range(len(list)):
            key = str.upper((s.join(list[i:])))
            keys.append(key)
            if key in self.index:
                pointer = self.index[key]
                break
        else:
            i = len(list)
            pointer = None
        # Do it into temporaries first so exceptions don't
        # mess up self.index and self.buf
        buf = ''
        offset = len(self.buf)
        index = []
        for j in range(i):
            label = list[j]
            n = len(label)
            if offset + len(buf) < 0x3FFF:
                index.append((keys[j], offset + len(buf)))
            else:
                print('DNS.Lib.Packer.addname:',)
                print('warning: pointer too big')
            buf = buf + (chr(n) + label)
        if pointer:
            buf = buf + pack16bit(pointer | 0xC000)
        else:
            buf = buf + '\0'
        self.buf = self.buf + bytes(buf, encoding='utf-8')
        for key, value in index:
            self.index[key] = value

    def dump(self):
        keys = sorted(self.index.keys())
        print('-' * 40)
        for key in keys:
            print('%20s %3d' % (key, self.index[key]))
        print('-' * 40)
        space = 1
        for i in range(0, len(self.buf) + 1, 2):
            if self.buf[i:i + 2] == '**':
                if not space:
                    print()
                space = 1
                continue
            space = 0
            print('%4d' % i,)
            for c in self.buf[i:i + 2]:
                if ' ' < c < '\177':
                    print(' %c' % c,)
                else:
                    print('%2d' % ord(c),)
            print()
        print('-' * 40)


# Unpacking class


class Unpacker:

    def __init__(self, buf):
        self.buf = buf
        self.offset = 0

    def getbyte(self):
        if self.offset >= len(self.buf):
            raise UnpackError("Ran off end of data")
        c = self.buf[self.offset]
        self.offset = self.offset + 1
        return c

    def getbytes(self, n):
        s = self.buf[self.offset: self.offset + n]
        if len(s) != n:
            raise UnpackError('not enough data left')
        self.offset = self.offset + n
        return s

    def get16bit(self):
        return unpack16bit(self.getbytes(2))

    def get32bit(self):
        return unpack32bit(self.getbytes(4))

    def getaddr(self):
        return bin2addr(self.get32bit())

    def getstring(self):
        return self.getbytes(ord(self.getbyte()))

    def getname(self):
        # Domain name unpacking (section 4.1.4)
        c = self.getbyte()
        i = ord(chr(c))
        if i & 0xC0 == 0xC0:
            d = self.getbyte()
            j = ord(chr(d))
            pointer = ((i << 8) | j) & ~0xC000
            save_offset = self.offset
            try:
                self.offset = pointer
                domain = self.getname()
            finally:
                self.offset = save_offset
            return domain
        if i == 0:
            return ''
        domain = self.getbytes(i).decode('UTF-8')
        remains = self.getname()
        if not remains:
            return domain
        else:
           return domain + '.' + remains


# Test program for packin/unpacking (section 4.1.4)

def testpacker():
    N = 2500
    R = range(N)
    import timing
    # See section 4.1.4 of RFC 1035
    timing.start()
    for i in R:
        p = Packer()
        p.addaddr('192.168.0.1')
        p.addbytes('*' * 20)
        p.addname('f.ISI.ARPA')
        p.addbytes('*' * 8)
        p.addname('Foo.F.isi.arpa')
        p.addbytes('*' * 18)
        p.addname('arpa')
        p.addbytes('*' * 26)
        p.addname('')
    timing.finish()
    print(timing.milli(), "ms total for packing")
    print(round(timing.milli() / i, 4), 'ms per packing')
    # p.dump()
    u = Unpacker(p.buf)
    u.getaddr()
    u.getbytes(20)
    u.getname()
    u.getbytes(8)
    u.getname()
    u.getbytes(18)
    u.getname()
    u.getbytes(26)
    u.getname()
    timing.start()
    for i in R:
        u = Unpacker(p.buf)

        res = (u.getaddr(),
               u.getbytes(20),
               u.getname(),
               u.getbytes(8),
               u.getname(),
               u.getbytes(18),
               u.getname(),
               u.getbytes(26),
               u.getname())
    timing.finish()
    print(timing.milli(), "ms total for unpacking")
    print(round(timing.milli() / i, 4), 'ms per unpacking')
    # for item in res: print item


# Pack/unpack RR toplevel format (section 3.2.1)

class RRpacker(Packer):

    def __init__(self):
        Packer.__init__(self)
        self.rdstart = None

    def addRRheader(self, name, type, klass, ttl, *rest):
        self.addname(name)
        self.add16bit(type)
        self.add16bit(klass)
        self.add32bit(ttl)
        if rest:
            if rest[1:]:
                raise TypeError('too many args')
            rdlength = rest[0]
        else:
            rdlength = 0
        self.add16bit(rdlength)
        self.rdstart = len(self.buf)

    def patchrdlength(self):
        rdlength = unpack16bit(self.buf[self.rdstart - 2:self.rdstart])
        if rdlength == len(self.buf) - self.rdstart:
            return
        rdata = self.buf[self.rdstart:]
        save_buf = self.buf
        ok = 0
        try:
            self.buf = self.buf[:self.rdstart - 2]
            self.add16bit(len(rdata))
            self.buf = self.buf + rdata
            ok = 1
        finally:
            if not ok:
                self.buf = save_buf

    def endRR(self):
        if self.rdstart is not None:
            self.patchrdlength()
        self.rdstart = None

    def getbuf(self):
        if self.rdstart is not None:
            self.patchrdlength()
        return Packer.getbuf(self)
    # Standard RRs (section 3.3)

    def addCNAME(self, name, klass, ttl, cname):
        self.addRRheader(name, Type.CNAME, klass, ttl)
        self.addname(cname)
        self.endRR()

    def addHINFO(self, name, klass, ttl, cpu, os):
        self.addRRheader(name, Type.HINFO, klass, ttl)
        self.addstring(cpu)
        self.addstring(os)
        self.endRR()

    def addMX(self, name, klass, ttl, preference, exchange):
        self.addRRheader(name, Type.MX, klass, ttl)
        self.add16bit(preference)
        self.addname(exchange)
        self.endRR()

    def addNS(self, name, klass, ttl, nsdname):
        self.addRRheader(name, Type.NS, klass, ttl)
        self.addname(nsdname)
        self.endRR()

    def addPTR(self, name, klass, ttl, ptrdname):
        self.addRRheader(name, Type.PTR, klass, ttl)
        self.addname(ptrdname)
        self.endRR()

    def addSOA(self, name, klass, ttl,
               mname, rname, serial, refresh, retry, expire, minimum):
        self.addRRheader(name, Type.SOA, klass, ttl)
        self.addname(mname)
        self.addname(rname)
        self.add32bit(serial)
        self.add32bit(refresh)
        self.add32bit(retry)
        self.add32bit(expire)
        self.add32bit(minimum)
        self.endRR()

    def addTXT(self, name, klass, ttl, list):
        self.addRRheader(name, Type.TXT, klass, ttl)
        if isinstance(list, str):
            list = [list]
        for txtdata in list:
            self.addstring(txtdata)
        self.endRR()
    # Internet specific RRs (section 3.4) -- class = IN

    def addA(self, name, klass, ttl, address):
        self.addRRheader(name, Type.A, klass, ttl)
        self.addaddr(address)
        self.endRR()

    def addWKS(self, name, ttl, address, protocol, bitmap):
        self.addRRheader(name, Type.WKS, Class.IN, ttl)
        self.addaddr(address)
        self.addbyte(chr(protocol))
        self.addbytes(bitmap)
        self.endRR()

    def addSRV(self):
        raise NotImplementedError


def prettyTime(seconds):
    if seconds < 60:
        return seconds, "%d seconds" % (seconds)
    if seconds < 3600:
        return seconds, "%d minutes" % (seconds / 60)
    if seconds < 86400:
        return seconds, "%d hours" % (seconds / 3600)
    if seconds < 604800:
        return seconds, "%d days" % (seconds / 86400)
    else:
        return seconds, "%d weeks" % (seconds / 604800)


class RRunpacker(Unpacker):

    def __init__(self, buf):
        Unpacker.__init__(self, buf)
        self.rdend = None

    def getRRheader(self):
        name = self.getname()
        rrtype = self.get16bit()
        klass = self.get16bit()
        ttl = self.get32bit()
        rdlength = self.get16bit()
        self.rdend = self.offset + rdlength
        return (name, rrtype, klass, ttl, rdlength)

    def endRR(self):
        if self.offset != self.rdend:
            raise UnpackError('end of RR not reached')

    def getCNAMEdata(self):
        return self.getname()

    def getHINFOdata(self):
        return self.getstring(), self.getstring()

    def getMXdata(self):
        return self.get16bit(), self.getname()

    def getNSdata(self):
        return self.getname()

    def getPTRdata(self):
        return self.getname()

    def getSOAdata(self):
        return self.getname(), \
            self.getname(), \
            ('serial',) + (self.get32bit(),), \
            ('refresh ',) + prettyTime(self.get32bit()), \
            ('retry',) + prettyTime(self.get32bit()), \
            ('expire',) + prettyTime(self.get32bit()), \
            ('minimum',) + prettyTime(self.get32bit())

    def getTXTdata(self):
        list = []
        while self.offset != self.rdend:
            list.append(self.getstring())
        return list

    def getAdata(self):
        return self.getaddr()

    def getWKSdata(self):
        address = self.getaddr()
        protocol = ord(self.getbyte())
        bitmap = self.getbytes(self.rdend - self.offset)
        return address, protocol, bitmap

    def getSRVdata(self):
        """
        _Service._Proto.Name TTL Class SRV Priority Weight Port Target
        """
        priority = self.get16bit()
        weight = self.get16bit()
        port = self.get16bit()
        target = self.getname()
        # print '***priority, weight, port, target', priority, weight, port,
        # target
        return priority, weight, port, target


# Pack/unpack Message Header (section 4.1)

class Hpacker(Packer):

    def addHeader(self, id, qr, opcode, aa, tc, rd, ra, z, rcode,
                  qdcount, ancount, nscount, arcount):
        self.add16bit(id)
        self.add16bit((qr & 1) << 15 | (opcode & 0xF) << 11 | (aa & 1) << 10
                      | (tc & 1) << 9 | (rd & 1) << 8 | (ra & 1) << 7
                      | (z & 7) << 4 | (rcode & 0xF))
        self.add16bit(qdcount)
        self.add16bit(ancount)
        self.add16bit(nscount)
        self.add16bit(arcount)


class Hunpacker(Unpacker):

    def getHeader(self):
        id = self.get16bit()
        flags = self.get16bit()
        qr, opcode, aa, tc, rd, ra, z, rcode = (
            (flags >> 15) & 1,
            (flags >> 11) & 0xF,
            (flags >> 10) & 1,
            (flags >> 9) & 1,
            (flags >> 8) & 1,
            (flags >> 7) & 1,
            (flags >> 4) & 7,
            (flags >> 0) & 0xF)
        qdcount = self.get16bit()
        ancount = self.get16bit()
        nscount = self.get16bit()
        arcount = self.get16bit()
        return (id, qr, opcode, aa, tc, rd, ra, z, rcode,
                qdcount, ancount, nscount, arcount)


# Pack/unpack Question (section 4.1.2)

class Qpacker(Packer):

    def addQuestion(self, qname, qtype, qclass):
        self.addname(qname)
        self.add16bit(qtype)
        self.add16bit(qclass)


class Qunpacker(Unpacker):

    def getQuestion(self):
        return self.getname(), self.get16bit(), self.get16bit()


# Pack/unpack Message(section 4)
# NB the order of the base classes is important for __init__()!

class Mpacker(RRpacker, Qpacker, Hpacker):
    pass


class Munpacker(RRunpacker, Qunpacker, Hunpacker):
    pass


# Routines to print an unpacker to stdout, for debugging.
# These affect the unpacker's current position!

def dumpM(u):
    print('HEADER:',)
    (id, qr, opcode, aa, tc, rd, ra, z, rcode,
     qdcount, ancount, nscount, arcount) = u.getHeader()
    print('id=%d,' % id,)
    print('qr=%d, opcode=%d, aa=%d, tc=%d, rd=%d, ra=%d, z=%d, rcode=%d,' \
        % (qr, opcode, aa, tc, rd, ra, z, rcode))
    if tc:
        print('*** response truncated! ***')
    if rcode:
        print('*** nonzero error code! (%d) ***' % rcode)
    print('  qdcount=%d, ancount=%d, nscount=%d, arcount=%d' \
        % (qdcount, ancount, nscount, arcount))
    for i in range(qdcount):
        print('QUESTION %d:' % i,)
        dumpQ(u)
    for i in range(ancount):
        print('ANSWER %d:' % i,)
        dumpRR(u)
    for i in range(nscount):
        print('AUTHORITY RECORD %d:' % i,)
        dumpRR(u)
    for i in range(arcount):
        print('ADDITIONAL RECORD %d:' % i,)
        dumpRR(u)


class DnsResult:

    def __init__(self, u, args):
        self.header = {}
        self.questions = []
        self.answers = []
        self.authority = []
        self.additional = []
        self.args = args
        self.storeM(u)

    def show(self):
        import time
        print('; <<>> PDG.py 1.0 <<>> %s %s' % (self.args['name'],
                                                self.args['qtype']))
        opt = ""
        if self.args['rd']:
            opt = opt + 'recurs '
        h = self.header
        print(';; options: ' + opt)
        print(';; got answer:')
        print(';; ->>HEADER<<- opcode %s, status %s, id %d' % (
            h['opcode'], h['status'], h['id']))
        flags = filter(lambda x, h=h: h[x], ('qr', 'aa', 'rd', 'ra', 'tc'))
        print(';; flags: %s; Ques: %d, Ans: %d, Auth: %d, Addit: %d' % (
            ''.join(map(str,flags)), h['qdcount'], h['ancount'], h['nscount'],
            h['arcount']))
        print(';; QUESTIONS:')
        for q in self.questions:
            print(';;      %s, type = %s, class = %s' % (q['qname'], q['qtypestr'],
                                                         q['qclassstr']))
        print()
        print(';; ANSWERS:')
        for a in self.answers:
            print('%-20s    %-6s  %-6s  %s' % (a['name'], repr(a['ttl']), a['typename'],
                                               a['data']))
        print()
        print(';; AUTHORITY RECORDS:')
        for a in self.authority:
            print('%-20s    %-6s  %-6s  %s' % (a['name'], repr(a['ttl']), a['typename'],
                                               a['data']))
        print()
        print(';; ADDITIONAL RECORDS:')
        for a in self.additional:
            print('%-20s    %-6s  %-6s  %s' % (a['name'], repr(a['ttl']), a['typename'],
                                               a['data']))
        print()
        if 'elapsed' in self.args:
            print(';; Total query time: %d msec' % self.args['elapsed'])
        print(';; To SERVER: %s' % (self.args['server']))
        print(';; WHEN: %s' % time.ctime(time.time()))

    def storeM(self, u):
        (self.header['id'], self.header['qr'], self.header['opcode'],
         self.header['aa'], self.header['tc'], self.header['rd'],
         self.header['ra'], self.header['z'], self.header['rcode'],
         self.header['qdcount'], self.header['ancount'],
         self.header['nscount'], self.header['arcount']) = u.getHeader()
        self.header['opcodestr'] = Opcode.opcodestr(self.header['opcode'])
        self.header['status'] = Status.statusstr(self.header['rcode'])
        for i in range(self.header['qdcount']):
            # print 'QUESTION %d:' % i,
            self.questions.append(self.storeQ(u))
        for i in range(self.header['ancount']):
            # print 'ANSWER %d:' % i,
            self.answers.append(self.storeRR(u))
        for i in range(self.header['nscount']):
            # print 'AUTHORITY RECORD %d:' % i,
            self.authority.append(self.storeRR(u))
        for i in range(self.header['arcount']):
            # print 'ADDITIONAL RECORD %d:' % i,
            self.additional.append(self.storeRR(u))

    def storeQ(self, u):
        q = {}
        q['qname'], q['qtype'], q['qclass'] = u.getQuestion()
        q['qtypestr'] = Type.typestr(q['qtype'])
        q['qclassstr'] = Class.classstr(q['qclass'])
        return q

    def storeRR(self, u):
        r = {}
        r['name'], r['type'], r['class'], r[
            'ttl'], r['rdlength'] = u.getRRheader()
        r['typename'] = Type.typestr(r['type'])
        r['classstr'] = Class.classstr(r['class'])
        # print 'name=%s, type=%d(%s), class=%d(%s), ttl=%d' \
        #      % (name,
        #        type, typename,
        #        klass, Class.classstr(class),
        #        ttl)
        mname = 'get%sdata' % r['typename']
        if hasattr(u, mname):
            r['data'] = getattr(u, mname)()
        else:
            r['data'] = u.getbytes(r['rdlength'])
        return r


def dumpQ(u):
    qname, qtype, qclass = u.getQuestion()
    print('qname=%s, qtype=%d(%s), qclass=%d(%s)' \
        % (qname,
           qtype, Type.typestr(qtype),
           qclass, Class.classstr(qclass)))


def dumpRR(u):
    name, type, klass, ttl, rdlength = u.getRRheader()
    typename = Type.typestr(type)
    print('name=%s, type=%d(%s), class=%d(%s), ttl=%d' \
        % (name,
           type, typename,
           klass, Class.classstr(klass),
           ttl))
    mname = 'get%sdata' % typename
    if hasattr(u, mname):
        print('  formatted rdata:', getattr(u, mname)())
    else:
        print('  binary rdata:', u.getbytes(rdlength))

if __name__ == "__main__":
    testpacker()
#
# $Log: Lib.py,v $
# Revision 1.11.2.3  2007/05/22 20:27:40  customdesigned
# Fix unpacker underflow.
#
# Revision 1.11.2.2  2007/05/22 20:25:53  customdesigned
# Use socket.inetntoa,inetaton.
#
# Revision 1.11.2.1  2007/05/22 20:20:39  customdesigned
# Mark utf-8 encoding
#
# Revision 1.11  2002/03/19 13:05:02  anthonybaxter
# converted to class based exceptions (there goes the python1.4 compatibility :)
#
# removed a quite gross use of 'eval()'.
#
# Revision 1.10  2002/03/19 12:41:33  anthonybaxter
# tabnannied and reindented everything. 4 space indent, no tabs.
# yay.
#
# Revision 1.9  2002/03/19 10:30:33  anthonybaxter
# first round of major bits and pieces. The major stuff here (summarised
# from my local, off-net CVS server :/ this will cause some oddities with
# the
#
# tests/testPackers.py:
#   a large slab of unit tests for the packer and unpacker code in DNS.Lib
#
# DNS/Lib.py:
#   placeholder for addSRV.
#   added 'klass' to addA, make it the same as the other A* records.
#   made addTXT check for being passed a string, turn it into a length 1 list.
#   explicitly check for adding a string of length > 255 (prohibited).
#   a bunch of cleanups from a first pass with pychecker
#   new code for pack/unpack. the bitwise stuff uses struct, for a smallish
#     (disappointly small, actually) improvement, while addr2bin is much
#     much faster now.
#
# DNS/Base.py:
#   added DiscoverNameServers. This automatically does the right thing
#     on unix/ win32. No idea how MacOS handles this.  *sigh*
#     Incompatible change: Don't use ParseResolvConf on non-unix, use this
#     function, instead!
#   a bunch of cleanups from a first pass with pychecker
#
# Revision 1.8  2001/08/09 09:08:55  anthonybaxter
# added identifying header to top of each file
#
# Revision 1.7  2001/07/19 07:50:44  anthony
# Added SRV (RFC 2782) support. Code from Michael Str�der.
#
# Revision 1.6  2001/07/19 07:39:18  anthony
# 'type' -> 'rrtype' in getRRheader(). Fix from Michael Str�der.
#
# Revision 1.5  2001/07/19 07:34:19  anthony
# oops. glitch in storeRR (fixed now).
# Reported by Bastian Kleineidam and by greg lin.
#
# Revision 1.4  2001/07/19 07:16:42  anthony
# Changed (opcode&0xF)<<11 to (opcode*0xF)<<11.
# Patch from Timothy J. Miller.
#
# Revision 1.3  2001/07/19 06:57:07  anthony
# cvs keywords added
#
#
